@@ -147,11 +147,14 @@ that depends on non-hashable arguments):
 
 $$memoize_uw
 
-Here i used the functools.update_wrapper_ utility, which has
+Here I used the functools.update_wrapper_ utility, which has
 been added in Python 2.5 expressly to simplify the definition of decorators
 (in older versions of Python you need to copy the function attributes
 ``__name__``, ``__doc__``, ``__module__`` and ``__dict__``
 from the original function to the decorated function by hand).
+Here is an example of usage:
+
+$$f1
 
 .. _functools.update_wrapper: https://docs.python.org/3/library/functools.html#functools.update_wrapper
 
@@ -492,7 +495,7 @@ the implementation of a cool decorator utility (``decorator_apply``).
 ``FunctionMaker`` provides a ``.create`` classmethod which
 takes as input the name, signature, and body of the function
 we want to generate as well as the execution environment
-were the function is generated by ``exec``. Here is an example:
+where the function is generated by ``exec``. Here is an example:
 
 .. code-block:: python
 
@@ -523,7 +526,6 @@ be added to the generated function:
  >>> print(f1.__source__)
  def f1(a, b):
      f(a, b)
- <BLANKLINE>
 
 ``FunctionMaker.create`` can take as first argument a string,
 as in the examples before, or a function. This is the most common
@@ -1004,9 +1006,49 @@ callable objects, nor on built-in functions, due to limitations of the
 ``inspect`` module in the standard library, especially for Python 2.X
 (in Python 3.5 a lot of such limitations have been removed).
 
-There is a restriction on the names of the arguments: for instance,
-if try to call an argument ``_call_`` or ``_func_``
-you will get a ``NameError``:
+There is a strange quirk when decorating functions that take keyword
+arguments, if one of such arguments has the same name used in the
+caller function for the first argument. The quirk was reported by
+David Goldstein and here is an example where it is manifest:
+
+.. code-block:: python
+
+   >>> @memoize
+   ... def getkeys(**kw):
+   ...     return kw.keys()
+   >>> getkeys(func='a') # doctest: +ELLIPSIS
+   Traceback (most recent call last):
+    ...
+   TypeError: _memoize() got multiple values for ... 'func'
+
+The error message looks really strange until you realize that
+the caller function `_memoize` uses `func` as first argument,
+so there is a confusion between the positional argument and the
+keywork arguments. The solution is to change the name of the
+first argument in `_memoize`, or to change the implementation as
+follows:
+
+.. code-block:: python
+
+   def _memoize(*all_args, **kw):
+       func = all_args[0]
+       args = all_args[1:]
+       if kw:  # frozenset is used to ensure hashability
+           key = args, frozenset(kw.items())
+       else:
+           key = args
+       cache = func.cache  # attribute added by memoize
+       if key not in cache:
+           cache[key] = func(*args, **kw)
+       return cache[key]
+
+We have avoided the need to name the first argument, so the problem
+simply disappears. This is a technique that you should keep in mind
+when writing decorators for functions with keyword arguments.
+
+On a similar tone, there is a restriction on the names of the
+arguments: for instance, if try to call an argument ``_call_`` or
+``_func_`` you will get a ``NameError``:
 
 .. code-block:: python
 
@@ -1230,6 +1272,11 @@ def _memoize(func, *args, **kw):
 
 
 def memoize(f):
+    """
+    A simple memoize implementation. It works by adding a .cache dictionary
+    to the decorated function. The cache will grow indefinitely, so it is
+    your responsability to clear it, if needed.
+    """
     f.cache = {}
     return decorate(f, _memoize)
 
